# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import wx
from colofon.util.patterns import Singleton
from colofon.client.conf import IMAGE_PATH
import os


class Bitmap:
    themeDir = "base"
    noProjectDir = "others"

    def __init__(self, key, fileName):
        self.key = key
        self.fileName = fileName
        self._cachedBitmaps = {}

    def project(self, width=22, height=22, resize=True):
        if not (width, height, resize) in self._cachedBitmaps:
            self._cachedBitmaps[(width, height, resize)] = self._projectImp(
                width, height, resize
            )
        return self._cachedBitmaps[(width, height, resize)]

    def _projectImp(self, width=22, height=22, resize=True):
        if resize:
            dir = self._getSizeDir(width, height)
            bitmapDir = os.path.join(IMAGE_PATH, self.themeDir, dir)
            ret = wx.Image(
                os.path.join(bitmapDir, self._makeFileName(width, height))
            ).ConvertToBitmap()
            return ret
        else:
            bitmapDir = os.path.join(IMAGE_PATH, self.themeDir, self.noProjectDir)
            return wx.Image(os.path.join(bitmapDir, self.fileName)).ConvertToBitmap()

    def _getSizeDir(self, width, height):
        return "%dx%d" % (width, height)

    def _makeFileName(self, width=None, height=None):
        if width and height:
            return "%d%s" % (width, self.fileName)
        else:
            return self.fileName


class BitmapManager(Singleton):
    _Singleton__instance = None

    def __init__(self):
        Singleton.__init__(self)
        wx.InitAllImageHandlers()

        self.imgDict = {
            # Búsquedas
            "toggleFilter": "mostrar ocultar filtro.png",
            "toggleBookCard": "mostrar ocultar ficha.png",
            "union": "unir busqueda.png",
            "anida": "anidar.png",
            "clean": "borrar.png",
            "extractHtmlView": "ampliar vista.png",
            "search": "buscar.png",
            "notFilter": "notFilter.png",
            "printResult": "imprimirBus.png",
            # Estándars
            "notes": "adjuntar.png",
            "newNote": "n nota.png",
            "edit": "editar.png",
            "copy": "copy.png",
            "delete": "eliminar.png",
            "logicDelete": "anular.png",
            "print": "imprimir.png",
            "printLote": "imprimirLote.png",
            "close": "cerrar.png",
            "attach": "attach.png",
            "customAttr": "customAttr.png",
            "mail": "mail.png",
            "calendar": "calendar.png",
            "new": "nuevo.png",
            "newBatch": "nuevolote.png",
            "newBatchPerItem": "nuevoloteporitem.png",
            "changeBatchItemStatus": "cambiarestadolote.png",
            "notes_red": "adjuntar.png",
            "cancel": "cancelar.png",
            "undo": "undo.png",
            "import": "importar.png",
            "export": "exportar.png",
            "exportAFIP": "exportarAfip.png",
            "exportIIBB": "exportarIIBB.png",
            "exportIIBBCaba": "exportarIIBB.png",
            "save": "guardar.png",
            "preview": "refrescar.png",
            "refresh": "refrescar.png",
            "lock": "bloquear.png",
            "assign": "flecha abajo.png",
            "run": "buscar.png",
            "validate": "validar.png",
            "noValidate": "no valido.png",
            "barcode": "barcode.png",
            "mail": "mail.png",
            "barcode": "barcode.png",
            "report": "report.png",
            "modernClock": "modernClock.png",
            # Cambios
            "changeQuantity": "cambiarCantidad.png",
            "changeStatus": "cambiarEstado.png",
            # Flechas de orden de las columnas
            "flechaColumnDown": "flechaColumnDown.png",
            "flechaColumnUp": "flechaColumnUp.png",
            "arrow_down": "flecha abajo.png",
            "arrow_up": "flecha arriba.png",
            "arrow_left": "flecha izquierda.png",
            "arrow_right": "flecha derecha.png",
            "arrow_first": "arrow_first.png",
            "arrow_last": "arrow_last.png",
            "columnDown": "flecha abajo.png",
            "columnUp": "flecha arriba.png",
            # Abm árbol
            "newChild": "crear hijo.png",
            "newSibling": "crear hermano.png",
            "sort_ascending": "orden ascendente.png",
            "sort_descending": "orden descendente.png",
            # Semáforo
            "redBall": "semaforo rojo.png",
            "greenBall": "semaforo verde.png",
            "yellowBall": "semaforo amarillo.png",
            # Imágenes del control Tree
            "check": "check.png",
            "uncheck": "uncheck.png",
            # Pantalla de login
            "expandPanel": "desplegar hacia abajo.png",
            "collapsePanel": "desplegar hacia arriba.png",
            "findServer": "refrescar.png",
            "login": "logo_trans.png",
            "authorize": "authorize.png",
            # Diálogo de error
            "error": "error.png",
            # Navegación
            "first": "flecha_izquierda_navegar.png",
            "last": "flecha_derecha_navegar.png",
            "next": "flecha derecha.png",
            "prev": "flecha izquierda.png",
            # Nuevos y usados
            "productTypeNew": "productTypeNew.png",
            "productTypeUsed": "productTypeUsed.png",
            "searchUsed": "findProductTypeUsed.png",
            "searchNew": "findProductTypeNew.png",
            # Caja y bancos
            # Aprobados
            "certifyBankMovement": "validar.png",
            "uncertifyBankMovement": "no valido.png",
            "editDate": "cambiar fecha.png",
            "editAmount": "cambiar importe.png",
            "bank": "banco.png",
            "cashBox": "caja.png",
            "depot": "deposito.png",
            "extraction": "extraccion.png",
            "extraction_cheque": "extraccion_cheque.png",
            "cashBoxMovement": "mov manual caja.png",
            "movementBetweenCashBoxes": "mov entre cajas.png",
            "cashChange": "cash change.png",
            "showCheck": "cheque.png",
            "cancelledCheck": "cheque anulado.png",
            "bankAccountMovement": "mov manual banco.png",
            "thirdPartCheck": "thirdPartCheck.png",
            "checkReplace": "n canjecheques.png",
            "processCreditCardAutomaticDebit": "presentarddaa.png",
            # Compras
            "provider": "proveedores.png",
            "providerWizard": "providerWizard.png",
            "importProvidersWizard": "importProvidersWizard.png",  # Controlar esto
            # Documentos
            "packet": "paquetes.png",
            "remit": "remitos.png",
            "remitReturn": "devoluciones.png",
            "bill": "facturas.png",
            "buyOrder": "pedidos.png",
            "billOrder": "pedido de baja.png",
            "credit": "credito.png",
            "creditNote": "nota de credito.png",
            "payOrder": "orden de pago.png",
            "receipt": "recibo.png",
            "expense": "gastos.png",
            "deposit": "boletasdeposito.png",
            "buyPreOrder": "buyPreOrder.png",
            # Nuevo Documento
            "newPacket": "n paquetes.png",
            "newRemit": "n remitos.png",
            "newRemitReturn": "n devoluciones.png",
            "newBillRemit": "n factura remito.png",
            "newBill": "n factura.png",
            "newGeneralBill": "n factura general.png",
            "newBuyOrder": "n pedidos.png",
            "newBillOrder": "n pedido de baja.png",
            "newCredit": "n credito.png",
            "newCreditGeneral": "n credito general.png",
            "newCreditNoteGeneral": "n nota de credito gra.png",
            "newCreditNote": "n nota de credito.png",
            "newPayOrder": "n orden de pago.png",
            "newReceipt": "n recibo.png",
            "newExpense": "n gastos.png",
            "newDeposit": "n boletadeposito.png",
            # 'newBuyOrder': 'newBuyOrder.png',
            # Buscar un Documento
            "findPacket": "b paquetes.png",
            "findRemit": "b remito.png",
            "findRemitReturn": "b devoluciones.png",
            "findBillRemit": "b factura remito.png",
            "findBill": "b factura.png",
            "findGeneralBill": "f factura general.png",
            "findBuyOrder": "b pedido.png",
            "findBillOrder": "b pedido de baja.png",
            "findCredit": "b credito.png",
            "findCreditGeneral": "b credito general.png",
            "findCreditNote": "b nota de credito.png",
            "findPayOrder": "b orden de pago.png",
            "findReceipt": "b recibo.png",
            "findExpense": "b gastos.png",
            "findDoc": "findDoc.png",
            "payment_cash": "efectivo.png",
            "edit_payments": "editarPagos.png",
            "payment_credit_card": "tarjeta.png",
            "payment_credit_card_automatic_debit": "debito_automatico.png",
            "payment_discount": "otros descuentos.png",
            "afip": "afip.png",
            "dian": "dian.png",
            "docSaleAccountPayment": "docSaleAccountPayment.png",
            "docBuyAccountPayment": "docSaleAccountPayment.png",
            "depotPayment": "depotPayment.png",
            "transferPayment": "transferPayment.png",
            "saleQuota": "saleQuota.png",
            "editCommission": "incrementPercentage.png",
            "saleQuotaDir": "saleQuotaDir.png",
            "cashOnDelivery": "correoArgentino.png",
            "universityOffice": "universityOffice.png",
            "universityEmployee": "universityEmployee.png",
            "closePayOrder": "cerrar orden de pago.png",
            "savePayOrderPayment": "cambiar importe.png",
            # Cuenta Corriente
            "buyAccountByDebits": "por debito.png",
            "buyAccountByCredits": "por credito.png",
            "buyAccountByDate": "por fecha.png",
            # Ventas
            "clientWizard": "clientWizard.png",
            "importClientsWizard": "clientImportWizard.png",
            "newBillSelectedBook": "facturar libro.png",
            "newAnticipeSelectedBook": "seniarLibro.png",
            "newReservationSelectedBook": "reservarLibro.png",
            "client": "clientes.png",
            "newClient": "newClient.png",
            # Docs
            "saleBill": "vta factura.png",
            "saleCreditNote": "vta nota de credito.png",
            "reservation": "vtareserva.png",
            "anticipe": "vtasenia.png",
            "newReservation": "nreserva.png",
            "newPhoneReservation": "newPhoneReservation.png",
            "newAnticipe": "nsenia.png",
            "newGeneralAnticipe": "_new_general_anticipe.png",
            "findReservation": "breserva.png",
            "saleReceipt": "vta_recibo.png",
            "newSaleReceipt": "n_vta_recibo.png",
            "findSaleReceipt": "vta_recibo.png",
            "saleOrder": "_sale_order.png",
            "newSaleOrder": "_new_sale_order.png",
            "findSaleOrder": "_find_sale_order.png",
            "newSaleBillNR": "_new_sale_bill_nr.png",
            "newSaleBillGeneral": "_new_sale_bill_general.png",
            "newSaleBillRemit": "_new_sale_bill_remit.png",
            "saleRemit": "_sale_remit.png",
            "newSaleRemit": "_new_sale_remit.png",
            "newSaleRemitGeneral": "_new_sale_remit_general.png",
            "findSaleRemit": "_find_sale_remit.png",
            "saleRemitReturn": "_sale_remit_return.png",
            "newSaleRemitReturn": "_new_sale_remit_return.png",
            "findSaleRemitReturn": "_find_sale_remit_return.png",
            "newSaleCreditNoteSaleAccount": "_new_sale_credit_note_sale_account.png",
            "newSaleSaleCreditNoteGeneral": "_new_sale_credit_note_general.png",
            "returnLot": "_return_lot.png",
            "box": "box.png",
            "findBox": "findBox.png",
            "newBox": "newBox.png",
            "boxDocument": "boxDocument.png",
            "subscription": "_subscription.png",
            "newSubscription": "subscription.png",
            # Nuevos
            "newSaleBill": "n vta factura.png",
            "newSaleCreditNote": "n vta nota de credito.png",
            "saleBillFromOther": "saleBillFromOther.png",
            # Buscar
            "findSaleBill": "b vta factura.png",
            "findSaleCreditNote": "b vta nota de credito.png",
            "salesmanWizard": "salesmanWizard.png",
            "employeWizard": "employeWizard.png",
            "gift": "gift.png",
            "points": "puntos.png",
            # Catálogo
            "book": "libro.png",
            "findBook": "b libro.png",
            "newBook": "n libro.png",
            "author": "autores.png",
            "price": "efectivo.png",
            "historicPrice": "precio historico.png",
            "importPriceWizard": "priceImportWizard.png",
            "bookSetWizard": "bookSetWizard.png",
            "documentImportWizard": "documentImportWizard.png",
            "saleOrderImportWizard": "saleOrderImportWizard.png",
            "inventoryAdjustmentWizard": "inventoryAdjustmentWizard.png",
            "bookWizard": "wizardLibro.jpg",
            "book-attachs": "book-attachments.png",
            "book-notes": "book-notes.png",
            "book-general": "book-general.png",
            "book-autores": "book-autores.png",
            "book-caracteristicas": "book-caracteristicas.png",
            "book-keywords": "book-keywords.png",
            "book-editorial": "book-editorial.png",
            "book-materias": "book-materias.png",
            "book-precio": "book-price.png",
            "book-proveedor": "book-provider.png",
            "book-ubicacion": "book-ubicacion.png",
            "incrementPercentage": "incrementPercentage.png",
            "decrementPercentage": "decrementPercentage.png",
            "round": "round.png",
            "task": "task.png",
            "newTask": "ntask.png",
            "resolveTask": "resolveTask.png",
            "editProvider": "editProvider.png",
            "logo-rilvi": "rilvi.png",
            "logo-cal": "cal.png",
            "logo-isbnEs": "isbnEs.png",
            "logo-aws": "aws.png",
            "importBooksWizard": "importBooksWizard.png",
            # Sistema
            "userGroupWizard": "userGroupWizard.png",
            "user": "usuario.png",
            "userGroup": "grupo de usuarios.png",
            "printerWizard": "printerWizard.png",
            # Stock
            "stockMovement": "mov stock.png",
            "newStockDocument": "doc stock.png",
            "findStockDocument": "b doc stock.png",
            "newInventoryCount": "inventario.png",
            "trayMessage": "trayMessage.png",
            "inboxMessage": "inboxMessage.png",
            "outboxMessage": "outboxMessage.png",
            "closeInventory": "cerrar_inventario.png",
            "reopenInventory": "reabrir_inventario.png",
            # WebPage
            "webPageWizard": "webPageWizard.png",
            "webPage": "web.png",
            "webUser": "webUser.png",
            "webBookComment": "webBookComment.png",
            "webAccept": "webAccept.png",
            "webReject": "webReject.png",
            "webSectionWizard": "webSectionWizard.png",
            "mailWebUser": "mailWebUser.png",
            "mailClient": "mailClient.png",
            "webUserWizard": "webUserWizard.png",
            # Common
            "transportWizard": "providerWizard.png",
            "redDelLibro": "redDelLibro.png",
            # Derechos de autor
            "rightHolderWizard": "rightHolderWizard.png",
            "rightHolder": "rightHolder.png",
            "contract": "contract.png",
            "contractWizard": "contractWizard.png",
            "liquidationWizard": "liquidationWizard.png",
            "newRHAnticipe": "n anticipo ddaa.png",
            # EditorHTML
            "htmlBackcolor": "html_backcolor.gif",
            "htmlBold": "html_bold.gif",
            "htmlBulList": "html_bullist.gif",
            "htmlFontDecrease": "html_font_decrease.gif",
            "htmlFontIncrease": "html_font_increase.gif",
            "htmlForecolor": "html_forecolor.gif",
            "htmlHR": "html_hr.gif",
            "htmlImage": "html_image.gif",
            "htmlIndent": "html_indent.gif",
            "htmlItalic": "html_italic.gif",
            "htmlJustifyCenter": "html_justifycenter.gif",
            "htmlJustifyFull": "html_justifyfull.gif",
            "htmlJustifyLeft": "html_justifyleft.gif",
            "htmlJustifyRight": "html_justifyright.gif",
            "htmlLink": "html_link.gif",
            "htmlNumList": "html_numlist.gif",
            "htmlOutdent": "html_outdent.gif",
            "htmlRemoveFormat": "html_removeformat.gif",
            "htmlStrikethrough": "html_strikethrough.gif",
            "htmlSub": "html_sub.gif",
            "htmlSup": "html_sup.gif",
            "htmlUnderline": "html_underline_es.gif",
            "htmlUnlink": "html_unlink.gif",
            # Módulo contable
            "entry": "entry.png",
            # Modulo Prensa
            "journalistWizard": "journalistWizard.png",
            "journalist": "journalist.png",
            # Nueva ventana de error
            "errorIcon": "errorIcon.png",
        }
        self._bitmapCache = {}

    def getBitmap(self, bitmapId):
        if bitmapId not in self._bitmapCache:
            self._bitmapCache[bitmapId] = Bitmap(bitmapId, self.imgDict[bitmapId])
        return self._bitmapCache[bitmapId]

    def getAppIcon(self):
        bitmapDir = os.path.join(IMAGE_PATH, Bitmap.themeDir, Bitmap.noProjectDir)
        return wx.Icon(os.path.join(bitmapDir, "icono.ico"), wx.BITMAP_TYPE_ICO)
